from django.contrib.sites.models import Site
from django.db import models
from django.conf import settings
from garpix_page.utils.all_sites import get_all_sites
from garpix_utils.file import get_file_path
from mptt.models import MPTTModel, TreeForeignKey
from garpix_utils.managers import ActiveManager
from ..mixins import LinkMixin
from ..validators import validate_type, validate_size
from garpix_utils.managers import GCurrentSiteManager


class MenuItem(LinkMixin, MPTTModel):
    """
    Пункты меню.
    """
    title_for_admin = models.CharField(max_length=100, blank=True, default='', verbose_name='Название для админа')
    title = models.CharField(max_length=100, verbose_name='Название')
    icon = models.FileField(
        upload_to=get_file_path, verbose_name='Иконка', blank=True, null=True,
        validators=[validate_type, validate_size]
    )
    menu_type = models.CharField(default='', max_length=100, choices=settings.CHOICE_MENU_TYPES, verbose_name='Тип меню')
    is_active = models.BooleanField(default=True, verbose_name='Включено')
    target_blank = models.BooleanField(default=False, verbose_name='Открывать в новом окне')
    created_at = models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')
    updated_at = models.DateTimeField(auto_now=True, verbose_name='Дата изменения')
    sort = models.IntegerField(default=100, verbose_name='Сортировка', help_text='Чем меньше число, тем выше будет элемент в списке.')
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children', db_index=True, verbose_name='Родительский пункт меню', on_delete=models.CASCADE)
    sites = models.ManyToManyField(Site, default=get_all_sites, verbose_name='Сайты для отображения')
    is_current = False
    is_current_full = False
    active_manager = ActiveManager()
    on_site = GCurrentSiteManager()

    class Meta:
        verbose_name = 'Пункт меню'
        verbose_name_plural = 'Пункты меню'
        ordering = ('sort', )
        abstract = False

    def __str__(self):
        if self.title_for_admin:
            return self.title_for_admin
        return self.title

    def get_menu_params(self):
        return settings.MENU_TYPES[self.menu_type]



from setuptools import setup, find_packages
from os import path

here = path.join(path.abspath(path.dirname(__file__)), 'garpix_menu')

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='garpix_menu',
    version='1.11.0',
    description='',
    long_description=long_description,
    author='Garpix LTD',
    author_email='info@garpix.com',
    license='MIT',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django >= 1.11',
        'django-mptt >= 0.9.1',
        'django-modeltranslation >= 0.16.2',
        'garpix_page >= 2.25.0',
        'garpix_utils >= 1.5.0'
    ],
)



from django.db import models
from django.utils import translation
from django.conf import settings
from garpix_page.models import BasePage
from garpix_utils.file import get_file_path


class LinkMixin(models.Model):
    """
    Пункты меню.
    """
    page = models.ForeignKey(BasePage, null=True, blank=True, verbose_name='Страница, на которую ведет пункт меню', help_text='Если этот пункт не выбран, то будет использовано следующее поле "Внешний URL"', on_delete=models.CASCADE)
    url = models.CharField(max_length=1000, null=True, blank=True, verbose_name='Внешний URL')
    hash = models.CharField(max_length=256, default='', blank=True, verbose_name='Якорь', help_text='Если хотите дать ссылку на конкретный элемент страницы. Например - #example')
    file = models.FileField(upload_to=get_file_path, blank=True, null=True, verbose_name='Файл')

    class Meta:
        abstract = True

    def get_link(self):
        if self.file not in (None, ''):
            return self.file.url
        elif self.page is not None:
            return f"{self.page.get_absolute_url()}{self.hash}"
        elif self.url is not None and self.url != '':
            if self.url.startswith('/'):
                current_language_code_url_prefix = translation.get_language()
                if current_language_code_url_prefix == settings.LANGUAGE_CODE:
                    return "{}{}".format(self.url, self.hash)
                return "/{}{}{}".format(current_language_code_url_prefix, self.url, self.hash)
            else:
                return self.url
        elif self.hash is not None and self.hash != '':
            return self.hash
        else:
            return '#'
    get_link.short_description = 'URL'



from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from django.conf import settings


def validate_type(value):
    exp = value.name.rsplit('.')[-1]
    if exp not in settings.MENU_ICON_ALLOWED_TYPES:
        alowed_types_str = ', '.join(settings.MENU_ICON_ALLOWED_TYPES)
        raise ValidationError(
            _('Допустимые типы') + " - " + alowed_types_str,
            params={'value': value},
        )


def validate_size(value):
    max_size = settings.MENU_ICON_MAX_SIZE / 1024 / 1024
    print(settings.MENU_ICON_MAX_SIZE, max_size, value.size)
    if value.size > settings.MENU_ICON_MAX_SIZE:
        raise ValidationError(
            _(f'Допустимый размер не более {max_size} МБ'),
            params={'value': value},
        )



from django.conf import settings
from django.forms.models import model_to_dict
from garpix_menu.models import MenuItem


def check_is_home(url):
    urls = ['', '/']
    for item in settings.LANGUAGES:
        urls.append(f'/{item[0]}')
    return url in urls


def get_menus(current_path):
    current_path_without_slash = current_path
    if current_path_without_slash[-1] == '/':
        current_path_without_slash = current_path_without_slash[0:-1]

    menus = {}
    for menu_type_arr in settings.CHOICE_MENU_TYPES:
        menu = MenuItem.on_site.filter(is_active=True, menu_type=menu_type_arr[0], parent=None)
        menus[menu_type_arr[0]] = []
        for menu_item in menu.order_by('sort', 'title'):
            link = menu_item.get_link()
            if link in (current_path, current_path_without_slash):
                menu_item.is_current = True
                menu_item.is_current_full = True
            elif current_path.startswith(link):
                if not check_is_home(link):
                    menu_item.is_current = True
            elif menu_item.url and menu_item.url.endswith(current_path):
                if not check_is_home(link):
                    menu_item.is_current = True
            menus[menu_type_arr[0]].append(model_to_dict(menu_item))
            menus[menu_type_arr[0]][-1]['get_link'] = link
            menus[menu_type_arr[0]][-1]['is_current'] = menu_item.is_current
            menus[menu_type_arr[0]][-1]['is_current_full'] = menu_item.is_current_full
            try:
                menus[menu_type_arr[0]][-1]['icon'] = menu_item.icon.url
            except Exception:
                menus[menu_type_arr[0]][-1]['icon'] = None
            menus[menu_type_arr[0]][-1].pop('page', None)
            menus[menu_type_arr[0]][-1].pop('title_for_admin', None)
    return menus



"""
Контекстный процессор для меню.
"""
from .utils import get_menus


def menu_processor(request):
    """
    Контекстный процессор для возможности отображения всех меню на сайте.
    Меню обычно распологаются на нескольких страницах, поэтому вынесено сюда.
    """
    current_path = request.path
    context = {
        'menus': get_menus(current_path),
    }
    return context
